@@ -5,7 +5,6 @@ Chemical formula parser.
 """
 from __future__ import division, print_function
 
-import sys
 from copy import copy
 from math import pi, sqrt
 
@@ -91,7 +90,7 @@ def _mix_by_weight_pairs(pairs):
     pairs = [(f, q) for f, q in pairs if q > 0]
 
     result = Formula()
-    if len(pairs) > 0:
+    if pairs:
         # cell mass = mass
         # target mass = q
         # cell mass * n = target mass
@@ -179,7 +178,7 @@ def _mix_by_volume_pairs(pairs):
             raise ValueError("Need the mass density of "+str(f))
 
     result = Formula()
-    if len(pairs) > 0:
+    if pairs:
         # cell volume = mass/density
         # target volume = q
         # cell volume * n = target volume
@@ -223,13 +222,13 @@ def formula(compound=None, density=None, natural_density=None,
         *ValueError* : invalid formula initializer
 
     After creating a formula, a rough estimate of the density can be
-    computed using:
+    computed using::
 
-         formula.density = formula.molecular_mass/formula.volume(packing_factor=...)
+        formula.density = formula.molecular_mass/formula.volume(packing_factor=...)
 
     The volume() calculation uses the covalent radii of the components and
     the known packing factor or crystal structure name.  If the lattice
-    constants for the crystal are known, then they can be used instead:
+    constants for the crystal are known, then they can be used instead::
 
         formula.density = formula.molecular_mass/formula.volume(a, b, c, alpha, beta, gamma)
 
@@ -252,19 +251,22 @@ def formula(compound=None, density=None, natural_density=None,
         structure = _convert_to_hill_notation(compound)
     elif _is_string_like(compound):
         if ':' in compound:
+            # TODO: avoid circular imports
+            # TODO: support other biochemicals (carbohydrate residues, lipids)
             from . import fasta
             seq_type, seq = compound.split(':', 1)
             if seq_type in fasta.CODE_TABLES:
-                return fasta.Sequence(name=None, sequence=seq, type=seq_type).Hnatural
+                seq = fasta.Sequence(name=None, sequence=seq, type=seq_type)
+                return seq.labile_formula
         try:
-            formula = parse_formula(compound, table=table)
+            chem = parse_formula(compound, table=table)
             if name:
-                formula.name = name
+                chem.name = name
             if density is not None:
-                formula.density = density
+                chem.density = density
             elif natural_density is not None:
-                formula.natural_density = natural_density
-            return formula
+                chem.natural_density = natural_density
+            return chem
         except ValueError as exception:
             raise ValueError(str(exception))
             #print "parsed", compound, "as", self
@@ -404,7 +406,7 @@ class Formula(object):
         """
         return self.density
 
-    def volume(self, packing_factor='hcp', *args, **kw):
+    def volume(self, *args, **kw):
         r"""
         Estimate unit cell volume.
 
@@ -452,7 +454,17 @@ class Formula(object):
             formula.density = n*formula.molecular_mass/formula.volume()
 
         where n is the number of molecules per unit cell.
+
+        Note: a single non-keyword argument is interpreted as a packing factor
+        rather than a lattice spacing of 'a'.
         """
+        # Get packing factor
+        if len(args) == 1 and not kw:
+            packing_factor = args[0]
+            args = []
+        else:
+            packing_factor = kw.pop('packing_factor', 'hcp')
+
         # Let cell_volume sort out its own parameters.
         if args or kw:
             return cell_volume(*args, **kw)*1e-24
@@ -534,6 +546,20 @@ class Formula(object):
         self.structure = _change_table(self.structure, table)
         return self
 
+    def replace(self, source, target, portion=1):
+        """
+        Create a new formula with one atom/isotope substituted for another.
+
+        *formula* is the formula being updated.
+
+        *source* is the isotope/element to be substituted.
+
+        *target* is the replacement isotope/element.
+
+        *portion* is the proportion of source which is substituted for target.
+        """
+        return _isotope_substitution(self, source, target, portion=portion)
+
     def __eq__(self, other):
         """
         Return True if two formulas represent the same structure. Note
@@ -542,8 +568,7 @@ class Formula(object):
         """
         if not isinstance(other, Formula):
             return False
-        else:
-            return self.structure == other.structure
+        return self.structure == other.structure
 
     def __add__(self, other):
         """
@@ -588,6 +613,33 @@ class Formula(object):
         return "formula('%s')"%(str(self))
 
 
+def _isotope_substitution(compound, source, target, portion=1):
+    """
+    Substitute one atom/isotope in a formula with another in some proportion.
+
+    *compound* is the formula being updated.
+
+    *source* is the isotope/element to be substituted.
+
+    *target* is the replacement isotope/element.
+
+    *portion* is the proportion of source which is substituted for target.
+    """
+    atoms = compound.atoms
+    if source in atoms:
+        mass = compound.mass
+        mass_reduction = atoms[source]*portion*(source.mass - target.mass)
+        density = compound.density * (mass - mass_reduction)/mass
+        atoms[target] = atoms.get(target, 0) + atoms[source]*portion
+        if portion == 1:
+            del atoms[source]
+        else:
+            atoms[source] *= 1-portion
+    else:
+        density = compound.density
+    return formula(atoms, density=density)
+
+
 LENGTH_UNITS = {'nm': 1e-9, 'um': 1e-6, 'mm': 1e-3, 'cm': 1e-2}
 MASS_UNITS = {'ng': 1e-9, 'ug': 1e-6, 'mg': 1e-3, 'g': 1e+0, 'kg': 1e+3}
 VOLUME_UNITS = {'nL': 1e-9, 'uL': 1e-6, 'mL': 1e-3, 'L': 1e+0}
@@ -852,16 +904,14 @@ def _immutable(seq):
     """
     if isatom(seq):
         return seq
-    else:
-        return tuple((count+0, _immutable(fragment)) for count, fragment in seq)
+    return tuple((count+0, _immutable(fragment)) for count, fragment in seq)
 
 def _change_table(seq, table):
     """Converts lists to tuples so that structure is immutable."""
     if isatom(seq):
         return change_table(seq, table)
-    else:
-        return tuple((count, _change_table(fragment, table))
-                     for count, fragment in seq)
+    return tuple((count, _change_table(fragment, table))
+                 for count, fragment in seq)
 
 def _hill_compare(a, b):
     """
