@@ -93,23 +93,27 @@ associated with the periodictable package.
     Neutron Scattering Lengths in ILL
     Neutron Data Booklet (second edition), A.-J. Dianoux, G. Lander, Eds.
     Old City Publishing, Philidelphia, PA. pp 1.1-1 to 1.1-17.
+    (https://www.ill.eu/fileadmin/user_upload/ILL/1_About_ILL/Documentation/NeutronDataBooklet.pdf)
 
 .. [#Rauch2000] Rauch, H. and Waschkowski, W. (2000)
-    Neutron scattering lengths. Schopper, H. (ed.).
-    SpringerMaterials - The Landolt-Börnstein Database (http://www.springermaterials.com).
+    Neutron scattering lengths. Schopper, H. (ed.). SpringerMaterials -
+    The Landolt-Börnstein Database (http://www.springermaterials.com).
     doi: 10.1007/10499706_6
 
 .. [#Koester1991] Koester, L., Rauch, H., Seymann. E. (1991)
-    Atomic Data Nuclear Data Tables 49, 65
+    Atomic Data Nuclear Data Tables 49, 65. doi:10.1016/0092-640X(91)90012-S
 
 .. [#Lynn1990] Lynn, J.E. and Seeger, P.A. (1990)
     Resonance effects in neutron scattering lengths of rare-earth nuclides.
     Atomic Data and Nuclear Data Tables 44, 191-207.
+    doi:10.1016/0092-640X(90)90013-A
 
-.. [#Sears1999] Sears, V. F. (1999)
+.. [#Sears2006] Sears, V. F. (2006)
     4.4.4 Scattering lengths for neutrons.
-    In Wilson & Prince eds. Intl. Tables for Crystallography C
-    Kluwer Academic Publishers. pp 383.
+    In Prince, E. ed. Intl. Tables for Crystallography C.
+    Kluwer Academic Publishers. pp 444-454.
+    (https://it.iucr.org/Cb/ch4o4v0001/sec4o4o4/)
+    doi: 10.1107/97809553602060000103
 
 .. [#Sears1992] Sears, V.F. (1992)
     Neutron scattering lengths and cross sections.
@@ -117,18 +121,20 @@ associated with the periodictable package.
 
 .. [#May1982] May, R.P.,  Ibel, K. and Haas, J. (1982)
     The forward scattering of cold neutrons by mixtures of light and heavy water.
-    J. Appl. Cryst. 15, 15-19.
+    J. Appl. Cryst. 15, 15-19. doi:10.1107/S0021889882011285
+
+.. [#Mildner1998] Mildner, D.F.R., Lamaze, G.P. (1998)
+   Neutron Transmission of Single-Crystal Sapphire.
+   J Appl Crystallogr 31, 835–840. doi:10.1107/S0021889898005846
 
 .. [#Smith2006] Smith, G.S. and Majkrzak, C.M. (2006)
     2.9 Neutron reflectometry.
-    In E. Prince ed. Intl. Tables for Crystallography C
-    Wiley InterScience. pp 126-146.
-    doi: 10.1107/97809553602060000584
+    In E. Prince ed. Intl. Tables for Crystallography C.
+    Wiley InterScience. pp 126-146. doi: 10.1107/97809553602060000584
 
 .. [#Glinka2011] Glinka, C.J. (2011)
     Incoherent Neutron Scattering from Multi-element Materials.
-    J. Appl. Cryst. 44, 618-624.
-    doi: 10.1107/S0021889811008223
+    J. Appl. Cryst. 44, 618-624. doi: 10.1107/S0021889811008223
 """
 from __future__ import print_function
 
@@ -419,7 +425,7 @@ class Neutron(object):
 
         number_density = self._number_density*1e-24
 
-        # PAK 2017-04-21: compute incoherent cross section from total cross section
+        # PAK 2017-04-21: compute incoherent xs from total xs
         sigma_c = 4*pi/100 * b_c**2
         sigma_i = max(sigma_s - sigma_c, 0.)
 
@@ -466,7 +472,7 @@ class Neutron(object):
 
         number_density = self._number_density*1e-24
 
-        # PAK 2017-04-21: compute incoherent cross section from total cross section
+        # PAK 2017-04-21: compute incoherent xs from total xs
         sigma_c = 4*pi/100 * b_c**2
         sigma_i = max(sigma_s - sigma_c, 0.)
 
@@ -571,14 +577,16 @@ def init(table, reload=False):
     # Xe total cross section is missing from the table
     # put it in even though it has not been independently measured
     if table.Xe.neutron.total is None:
-        table.Xe.neutron.total = table.Xe.neutron.coherent + table.Xe.neutron.incoherent
+        table.Xe.neutron.total = (
+            table.Xe.neutron.coherent + table.Xe.neutron.incoherent)
 
 
+# TODO: require parsed compound rather than including formula() keywords in api
 # Note: docs and function prototype are reproduced in __init__
 @require_keywords
 def neutron_scattering(compound, density=None,
-                       wavelength=ABSORPTION_WAVELENGTH, energy=None,
-                       natural_density=None):
+                       wavelength=None, energy=None,
+                       natural_density=None, table=None):
     r"""
     Computes neutron scattering cross sections for molecules.
 
@@ -590,9 +598,11 @@ def neutron_scattering(compound, density=None,
         *natural_density* : float | |g/cm^3|
             Mass density of formula with naturally occuring abundances
         *wavelength* 1.798 : float | |Ang|
-            Neutron wavelength.
+            Neutron wavelength (default=1.798 |Ang|).
         *energy* : float | meV
             Neutron energy.  If energy is specified then wavelength is ignored.
+        *table* : PeriodicTable
+            Alternate table to use when parsing *compound*.
 
     :Returns:
         *sld* : (float, float, float) | |1e-6/Ang^2|
@@ -671,8 +681,8 @@ def neutron_scattering(compound, density=None,
 
     Similarly, the absorption cross section $\sigma_a$, the incoherent cross
     section $\sigma_i$, and the total cross section $\sigma_s$ can be computed
-    from the corresponding cross sections of the constituent elements,\ [#Sears1999]_
-    already expressed in barns:
+    from the corresponding cross sections of the constituent
+    elements,\ [#Sears2006]_ already expressed in barns:
 
     .. math::
 
@@ -694,12 +704,13 @@ def neutron_scattering(compound, density=None,
     are only valid for 1.798 |Ang|.
 
     From the scattering cross sections, the scattering length for a material
-    $b = b' - i b''$ can be computed using the following relations:[#Sears1999]_
+    $b = b' - i b''$ can be computed using the following relations:[#Sears2006]_
 
     .. math::
 
         \sigma_c &= 4 \pi |b_c|^2 \\
-        \sigma_a &= \left. 4 \pi \left< b'' \right> \right/k \ {\rm for} \ k=2\pi / \lambda \\
+        \sigma_a &= \left. 4 \pi \left< b'' \right> \right/k
+            \ {\rm for} \ k=2\pi / \lambda \\
         \sigma_i &= 4 \pi |b_i|^2 \\
         \sigma_s &= 4 \pi \left< |b|^2 \right>
 
@@ -724,12 +735,13 @@ def neutron_scattering(compound, density=None,
         \rho_{\rm im}  &= -N b'' / 100 \\
         \rho_{\rm inc} &= 10 N b_i
 
-    with the factors of 10 chosen to give SLD in units of $\AA^{-2}$.  The
+    with the factors of 10 chosen to give SLD in units of $\AA^{-2}$. The
     resulting $\rho = \rho_{\rm re} + i \rho_{\rm im}$ can be used in the
-    scattering equations.  Treatment of the incoherent scattering $\rho_{\rm inc}$
-    will depend on the equation.  For example, it can be treated as an absorption
-    in specular reflectivity calculations since the incoherently scattered neutrons
-    are removed from the multilayer recurrence calculation.
+    scattering equations. Treatment of the incoherent scattering
+    $\rho_{\rm inc}$ will depend on the equation. For example, it can be
+    treated as an absorption in specular reflectivity calculations since the
+    incoherently scattered neutrons are removed from the multilayer
+    recurrence calculation.
 
     Similarly, scattering cross section includes number density:
 
@@ -741,7 +753,7 @@ def neutron_scattering(compound, density=None,
         \Sigma_{\rm s} &= N \sigma_s
 
 
-    The 1/e penetration depth *t_u* represents the the depth into the sample at
+    The 1/e penetration depth $t_u$ represents the the depth into the sample at
     which the unscattered intensity is reduced by a factor of $e$:
 
     .. math::
@@ -755,8 +767,12 @@ def neutron_scattering(compound, density=None,
     If you instead want to calculate the effective shielding of the sample,
     you should recalculate penetration depth without the coherent scattering.
 
-    Transmission rate can be computed from $e^{-d/t_u}$ for penetration i
-    depth $t_u$ and sample thickness $d$.
+    Transmission rate can be computed from $e^{-d/t_u}$ for penetration
+    depth $t_u$ and sample thickness $d$. This does not include many
+    real world effects, such as single phonon scattering\ [#Mildner1998]_
+    and forward scattering\ [#May1982]_, which result in measured
+    transmission significantly different from the values predicted from
+    nuclear properties alone.
 
     In general, the total scattering cross section is not the sum of the
     coherent and incoherent cross sections,
@@ -804,11 +820,14 @@ def neutron_scattering(compound, density=None,
     """
     from . import formulas
     compound = formulas.formula(compound, density=density,
-                                natural_density=natural_density)
+                                natural_density=natural_density, table=table)
     assert compound.density is not None, "scattering calculation needs density"
+    #print("sld", compound, compound.density)
     if energy is not None:
         wavelength = neutron_wavelength(energy)
-    assert wavelength is not None, "scattering calculation needs energy or wavelength"
+    # PAK: 1.5.3 wavelength now defaults to ABSORPTION_WAVELENGTH
+    if wavelength is None:
+        wavelength = ABSORPTION_WAVELENGTH
 
     # Sum over the quantities
     molar_mass = num_atoms = 0
@@ -872,10 +891,11 @@ def neutron_sld(*args, **kw):
         *natural_density* : float | |g/cm^3|
             Mass density of formula with naturally occuring abundances
         *wavelength* : float | |Ang|
-            Neutron wavelength.
+            Neutron wavelength (default=1.798 |Ang|).
         *energy* : float | meV
             Neutron energy.  If energy is specified then wavelength is ignored.
-
+        *table* : PeriodicTable
+            Alternate table to use when parsing *compound*.
     :Returns:
         *sld* : (float, float, float) | |1e-6/Ang^2|
             (*real*, -*imaginary*, *incoherent*) scattering length density.
@@ -892,12 +912,121 @@ def neutron_sld_from_atoms(*args, **kw):
     r"""
     .. deprecated:: 0.91
 
-        :func:`neutron_sld` now accepts dictionaries of \{atom\: count\} directly.
+        :func:`neutron_sld` accepts dictionaries of \{atom\: count\}.
 
     """
     return neutron_scattering(*args, **kw)[0]
 
 
+def D2O_match(compound, **kw):
+    """
+    Find the D2O contrast match point for the compound.
+
+    *wavelength* or *energy* select neutron wavelength or energy.
+
+    Additional keyword arguments (*density*, *natural_density*, *name*, *table*)
+    are passed to :func:`formulas.formula` when parsing the compound.
+
+    Returns *D2O_fraction* and *SLD* at match point.
+
+    See :func:`D2O_sld` for details on the calculation.
+
+    Note that the resulting fraction is only meaningful in [0, 1]. Beyond
+    100% you will need an additional constrast agent in the 100% D2O
+    solvent to increase the SLD enough to match.
+    """
+    H2O_sld, D2O_sld, Hsld, Dsld = _D2O_slds(compound, **kw)
+    # SLD(%Dsample + (1-%)Hsample) = SLD(%D2O + (1-%)H2O)
+    # => %SLD(Dsample) + (1-%)SLD(Hsample) = %SLD(D2O) + (1-%)SLD(H2O)
+    # => %(SLD(Dsample) - SLD(Hsample) + SLD(H2O) - SLD(D2O))
+    #      = SLD(H2O) - SLD(Hsample)
+    # => % = 100*(SLD(H2O) - SLD(Hsample))
+    #      / (SLD(Dsample) - SLD(Hsample) + SLD(H2O) - SLD(D2O))
+    D2O_fraction = \
+        (H2O_sld[0] - Hsld[0]) / (Dsld[0] - Hsld[0] + H2O_sld[0] - D2O_sld[0])
+
+    match_point_sld = mix_values(Dsld, Hsld, D2O_fraction)
+    return D2O_fraction, match_point_sld[0]
+
+def D2O_sld(compound, volume_fraction=1., D2O_fraction=0., **kw):
+    """
+    Compute the neutron SLD for a D2O contrast solution.
+
+    *compound* is a string or parsed formula object. Labile hydrogen should
+    be marked as H[1] in the formula. These will be substituted according to
+    %D2O in the solvent.
+
+    The D2O contrast mixture is assumed to be made using pure H2O (with
+    its natural H:D ratios) and pure D2O with no H present, so H[1] will be
+    substituted alternately with H and D when computing mixture SLD.
+    Solvent SLD is calculated using the density at 20 C.
+
+    Only the coherent scattering crosssection will be matched. Incoherent
+    and absorption crosssections are likely to be different for the compound
+    and the solvent, especially due to the large incoherent crosssection for
+    hydrogen.
+
+    Note that incoherent scattering does not mix linearly, so the incoherent
+    sld for the mixture will differ slightly from incoherent scattering
+    computed returned from a compound with the same isotope ratios.
+
+    *volume_fraction* is the portion by volume of solute in the solution.
+
+    *D2O_fraction* is the portion by volume of D2O in the solvent.
+
+    *wavelength* or *energy* to select neutron wavelength or energy.
+
+    Additional keyword arguments (*density*, *natural_density*, *name*, *table*)
+    are passed to :func:`formulas.formula` when parsing the compound.
+
+    Returns (real, imag, incoh) SLD.
+    """
+    # TODO: fix incoherent scattering so it is consistent with compound
+    # Need to compute sld from mixture rather than mixing parts
+    H2O_sld, D2O_sld, Hsld, Dsld = _D2O_slds(compound, **kw)
+    solvent_sld = mix_values(D2O_sld, H2O_sld, D2O_fraction)
+    solute_sld = mix_values(Dsld, Hsld, D2O_fraction)
+    solution_sld = mix_values(solute_sld, solvent_sld, volume_fraction)
+    #print(D2O_fraction, volume_fraction)
+    #print(compound, "solvent", solvent_sld)
+    #print(compound, "solute", solute_sld)
+    #print(compound, "solution", solution_sld)
+    return solution_sld
+
+
+def _D2O_slds(compound, **kw):
+    from . import formulas
+
+    # Water density at 20C; neutron wavelength doesn't matter.
+    sld_args = dict(
+        wavelength=kw.pop("wavelength", None),
+        energy=kw.pop("energy", None),
+        # Note: using get() rather than pop() for table since table can be a
+        # parameter for formula and for neutron_sld (which calls formula)
+        table=kw.get('table', None),
+    )
+    # TODO: use same table for solvent as solute?
+    H2O_sld = neutron_sld("H2O@0.9982n", **sld_args)
+    D2O_sld = neutron_sld("D2O@0.9982n", **sld_args)
+    mol = formulas.formula(compound, **kw)
+    # Be sure to pull H and H[1] from the table for the compound, otherwise
+    # the elements may not match in the substitution.
+    # TODO: include table in compound so parsed
+    table = default_table(kw.get('table', None))
+    labile_H, H, D = table.H[1], table.H, table.D
+    Hsld = neutron_sld(mol.replace(labile_H, H), **sld_args)
+    Dsld = neutron_sld(mol.replace(labile_H, D), **sld_args)
+
+    return H2O_sld, D2O_sld, Hsld, Dsld
+
+
+def mix_values(a, b, fraction):
+    """
+    Mix two tuples with floating point values according to fraction of a.
+    """
+    return tuple(aj*fraction + bj*(1-fraction) for aj, bj in zip(a, b))
+
+
 def _sum_piece(wavelength, compound):
     """
     Helper for neutron_composite_sld which precomputes quantities of interest
@@ -1022,7 +1151,7 @@ def sld_plot(table=None):
 #   respectively
 # c
 #   'E' if there is a strong energy dependency.
-#   '+/-' if separate b+/b- values are available [doesn't seem true -PAK]
+#   '+/-' if separate b+/b- values are available [PAK: doesn't seem true]
 # coherent, incoherent, total
 #   The coherent and incoherent scattering cross-sections in barns.
 # absorption
@@ -1489,7 +1618,7 @@ def sld_table(wavelength=1, table=None, isotopes=True):
                   %(el, el.mass, el.density, coh, jcoh, inc,
                     ' *' if el.neutron.is_energy_dependent else ''))
             if isotopes:
-                isos = [iso for iso in el if iso.neutron != None and iso.neutron.has_sld()]
+                isos = [iso for iso in el if iso.neutron is not None and iso.neutron.has_sld()]
             else:
                 isos = []
             for iso in isos:
@@ -1532,8 +1661,8 @@ def energy_dependent_table(table=None):
             dep += [str(el)]
         dep += [str(el)+'-'+str(iso.isotope)
                 for iso in el
-                if iso.neutron != None and iso.neutron.is_energy_dependent]
-        if len(dep) > 0:
+                if iso.neutron is not None and iso.neutron.is_energy_dependent]
+        if dep:
             print("    " + " ".join(dep))
 
 def _diff(iso, a, b, tol=0.01):
@@ -1612,7 +1741,6 @@ def absorption_comparison_table(table=None, tol=None):
     compare(lambda el: el.neutron.absorption,
             lambda el: -2000*el.neutron.b_c_i*ABSORPTION_WAVELENGTH,
             table=table, tol=tol)
-    return
 
 def coherent_comparison_table(table=None, tol=None):
     r"""
@@ -1759,4 +1887,4 @@ def main():
         print_scattering(c, wavelength)
 
 if __name__ == "__main__":
-    main()
\ No newline at end of file
+    main()
